<?
require_once ("defines.php");

// Indices into the server information string
define (SERVER_NAME, '0');
define (SEARCH_BASE, '1');
define (PRIV_DN,     '2');
define (PRIV_PASSWD, '3');

class LDAP {
	var $server;
	var $connection_handle;
	var $error_no;
	var $error_txt;
	var $dn;
	var $dn_passwd;
	var $search_dn;
	var $search_string;
	var $attributes_to_get;
	var $search_result;
	var $info;               // 2D information array returned by ldap_search
	var $debug;
	
	function LDAP($in_server) {
		$parts = split (":", $in_server);
		
		$this->server = $parts[SERVER_NAME];
		$this->search_dn = $parts[SEARCH_BASE];
		$this->dn = $parts[PRIV_DN];
		$this->dn_passwd = $parts[PRIV_PASSWD];
	}
	
	function Connect() {
		if ($this->debug) {
			$this->connection_handle = ldap_connect ($this->server, get_site_option('ldapServerPort'));
		}
		else {
			$this->connection_handle = @ldap_connect ($this->server, get_site_option('ldapServerPort'));
		}
		
		// Error connecting?
		if (!$this->connection_handle) {
			$return = false;
			$this->LogError();
		}
		else {
			$return = true;
			ldap_set_option($this->connection_handle, LDAP_OPT_PROTOCOL_VERSION, 3);
			if (!get_site_option('ldapLinuxWindows')) {
				//Windows Only
				ldap_set_option($this->connection_handle, LDAP_OPT_REFERRALS, 0);
			}
		}
		
		return $return;
	}
	
	function Bind() {
		$this->error_no = 0;
		$this->error_txt = "Success";
		$return = false;
		
		if ($this->connection_handle) {
			if ($this->debug) {
				if (ldap_bind ($this->connection_handle, $this->dn, $this->dn_passwd)) {
					$return = true;
				}
			}
			else {
				if (@ldap_bind ($this->connection_handle, $this->dn, $this->dn_passwd))
				$return = true;
			}
		}
	
		if (!$return) {
			$this->error_no = @ldap_errno ($this->connection_handle);
			$this->LogError();
		}
	
		return $return;
	}
	
	function Dock() {
		// First, connect to the LDAP server
		$result = $this->Connect();
		if (!$result) {
			$this->LogError();
			return false;
		}
	
		// Now bind as the user with enough rights to browse the "cn" attribute
		if (!$this->Bind()) {
			$this->LogError();
			return false;
		}
	
		return true;
	}
	
	function Disconnect() {
		if ($this->connection_handle) {
			if ($this->debug) {
				ldap_close ($this->connection_handle);
			}
			else {
				@ldap_close ($this->connection_handle);
			}
	
			$this->connection_handle = 0;
		}
	}
	
	function GetErrorNumber() {
		return $this->error_no;
	}
	
	function GetErrorText() {
		return ldap_err2str ($this->error_no);
	}
	
	function SetAccessDetails ($dn, $passwd) {
		$this->dn = $dn;
		$this->dn_passwd = $passwd;
	}
	
	function SetSearchCriteria ($search, $attrs) {
		$this->search_string = $search;
		$this->attributes_to_get = $attrs;
	}
	
	function Search() {
		if (($this->connection_handle) && ($this->dn)) {
			if ($this->debug) {
				$this->search_result = ldap_search ($this->connection_handle, $this->search_dn, $this->search_string, $this->attributes_to_get);
				$this->info = ldap_get_entries ($this->connection_handle, $this->search_result);
			}
			else {
				$this->search_result = @ldap_search ($this->connection_handle, $this->search_dn, $this->search_string, $this->attributes_to_get);
				$this->info = @ldap_get_entries ($this->connection_handle, $this->search_result);
			}
		}
	}
	
	function DebugOn() {
		$this->debug = true;
	}
	
	function DebugOff() {
		$this->debug = false;
	}
	
	function LogError() {
		$this->error_no = @ldap_errno ($this->connection_handle);
	}
	
	function GetLDAPInfo ($type) {
		if ($type == LDAP_INDEX_EMAIL) 			return $this->info[0]["mail"][0];
		if ($type == LDAP_INDEX_NAME) 			return $this->info[0]["givenname"][0]." ".$this->info[0]["sn"][0];
		if ($type == LDAP_INDEX_GIVEN_NAME) 	return $this->info[0]["givenname"][0];
		if ($type == LDAP_INDEX_SURNAME) 		return $this->info[0]["sn"][0];
		if ($type == LDAP_INDEX_PHONE) 			return $this->info[0]["phone"];
		if ($type == LDAP_INDEX_HOMEDIR) 		return $this->info[0]["homedirectory"][0];
		if ($type == LDAP_INDEX_MEMBER) 		return $this->info[0]["member"];
		if ($type == LDAP_INDEX_MACADDRESS) 	return $this->info[0]["zenwmmacaddress"];
		// When dealing with "uniqueMember", LDAP actually returns it as "member" - they're synonyms
		if ($type == LDAP_INDEX_UNIQUE_MEMBER) 	return $this->info[0][member];
		if ($type == LDAP_INDEX_DN) 			return $this->info[0]["dn"];
	}
}
?>
