<?php
/**
 * Adds a link to the CSS for the LDAP authentication administative options.
 * Simply echoes the HTML to link the stylesheet to the current document.
 * 
 * @return null - does not actively return a value
 */
function ldapShowOptionsCss() {
  echo "<link rel='stylesheet' href='" . get_settings('siteurl') . "/wp-content/mu-plugins/ldap_auth.css' media='screen' type='text/css' />";
}

/**
 * Displays the LDAP options panel and handles changes made to the LDAP
 * settings.  Retrieves all the LDAP options from WP via get_site_option, and
 * only updates them by checking for the existence of $_POST['ldapOptionsSave'].
 * 
 * @return null - does not actively return a value
 * @see ldap_addmenu
 */
function ldapOptionsPanel() {
  if($_POST['ldapOptionsSave']) {
    update_site_option('ldapAuth', $_POST['ldapOptionsAuth']);
    update_site_option('ldapCreateAcct', $_POST['ldapOptionsCreateAcct']);
    update_site_option('ldapLinuxWindows', $_POST['ldapOptionsLinuxWindows']);
    update_site_option('ldapServerAddr', $_POST['ldapOptionsServerAddr']);
    update_site_option('ldapServerPort', $_POST['ldapOptionsServerPort']);
    update_site_option('ldapServerOU', $_POST['ldapOptionsServerOU']);
    update_site_option('ldapServerCN', $_POST['ldapOptionsServerCN']);
    update_site_option('ldapServerPass', $_POST['ldapOptionsServerPass']);
    update_site_option('ldapDisableSignup', $_POST['ldapOptionsDisableSignup']);
    update_site_option('ldapSignupMessage', stripslashes($_POST['ldapSignupMessage']));
    update_site_option('ldapGetPasswordMessage', stripslashes($_POST['ldapGetPasswordMessage']));
    
    echo "<div class='updated'><p>Saved Options!</p></div>";
  }
  
  $defaultSignupMessage = <<<NoSignupMsg
<p><strong>Public sign-up has been disabled.</strong></p>

<p>If you already have an account, please <a href="wp-login.php">log in</a> and 
try again.</p>    
NoSignupMsg;

  
  $sysAdminEmail = get_site_option('admin_email');
  $defaultGetPasswordMessage = <<<GetPasswordMsg
Your account is tied to an account in the central directory.  You cannot 
retrieve your password via email.  Please contact the 
<a href="mailto:$sysAdminEmail">system administrator</a> for information on how 
to reset your password.
GetPasswordMsg;

  
  $ldapAuth               = get_site_option('ldapAuth');
  $ldapCreateAcct         = get_site_option('ldapCreateAcct');
  $ldapLinuxWindows       = get_site_option('ldapLinuxWindows');
  $ldapCookieMarker       = get_site_option('ldapCookieMarker');
  $ldapServerAddr         = get_site_option('ldapServerAddr');
  $ldapServerPort         = get_site_option('ldapServerPort');
  $ldapServerOU           = get_site_option('ldapServerOU');
  $ldapServerCN           = get_site_option('ldapServerCN');
  $ldapServerPass         = get_site_option('ldapServerPass');
  $ldapDisableSignup      = get_site_option('ldapDisableSignup');
  $ldapSignupMessage      = stripslashes(get_site_option('ldapSignupMessage', $defaultSignupMessage));
  $ldapGetPasswordMessage = stripslashes(get_site_option('ldapGetPasswordMessage', $defaultGetPasswordMessage));

  // default values to avoid PHP notices about unset values
  $tChecked = '';
  $fChecked = '';
  $tAcctChecked = '';
  $fAcctChecked = '';
  $tLinWin = '';
  $fLinWin = '';
  $tDisableSignup = '';
  $fDisableSignup = '';
  
  
  if($ldapAuth) {
    $tChecked = 'checked';
  }
  else {
    $fChecked = 'checked';
  }
  
  if($ldapCreateAcct) {
    $tAcctChecked = 'checked';
  }
  else {
    $fAcctChecked = 'checked';
  }
  
  if($ldapLinuxWindows) {
    $tLinWin = 'checked';
  }
  else {
    $fLinWin = 'checked';
  }

  if($ldapDisableSignup) {
    $tDisableSignup = 'checked';
  }
  else {
    $fDisableSignup = 'checked';
  }

  
  if (!is_numeric($ldapServerPort)) {
    $ldapServerPort = 389;
  }
  
  echo <<<LdapForm
  <div class="wrap">
  <h2>Ldap Authentication Options</h2>
  <form method="post" id="ldap_auth_options">
    <fieldset class="options">
    <legend>General Server Settings</legend>
    
    <div class="row">
      <span class="description">LDAP Server Address:</span>
      <span class="element">
        <input type='text' name='ldapOptionsServerAddr' value='$ldapServerAddr' style='width: 300px;' /><br />
        <em>The name or IP address of the LDAP server.</em>
      </span>
    </div>
    
    <div class="row">
      <span class="description">LDAP Server Port:</span>
      <span class="element">
        <input type='text' name='ldapOptionsServerPort' value='$ldapServerPort' style='width: 300px;' /><br />
        <em>Port Number of the LDAP server. (Default: Linux=389, Windows=3268)</em>
      </span>
    </div>
    
    <div class="row">
      <span class="description">Search DN:</span>
      <span class="element">
        <input type='text' name='ldapOptionsServerOU' value='$ldapServerOU' style='width: 450px;' /><br />
        <em>The base DN for carrying out LDAP searches.</em>
      </span>
    </div>
    
    <div class="row">
      <span class="description">User DN:</span>
      <span class="element">
        <input type='text' name='ldapOptionsServerCN' value='$ldapServerCN' style='width: 450px;' /><br />
        <em>Some systems do not allow anonymous searching for attributes.</em>
      </span>
    </div>
    
    <div class="row">
      <span class="description">Password:</span>
      <span class="element">
        <input type='password' name='ldapOptionsServerPass' value='$ldapServerPass' /><br />
        <em>Password for the privileged user.</em>
      </span>
    </div>
    
    <div class="row">
      <span class="description">Linux or Windows LDAP:</span><br />
      <span class="element">
        <input type='radio' name='ldapOptionsLinuxWindows' value='1' $tLinWin/> Linux
        <input type='radio' name='ldapOptionsLinuxWindows' value='0' $fLinWin/> Windows
      </span>
    </div>
    
    <div class="row">
      <span class="description">Auto Create Local Accounts:</span><br />
      <span class="element">
        <input type='radio' name='ldapOptionsCreateAcct' value='1' $tAcctChecked/> Yes
        <input type='radio' name='ldapOptionsCreateAcct' value='0' $fAcctChecked/> No
      </span>
    </div>
    
    <div class="row">
      <span class="description">Enable LDAP?</span><br />
      <span class="element">
        <input type='radio' name='ldapOptionsAuth' value='1' $tChecked/> Yes
        <input type='radio' name='ldapOptionsAuth' value='0' $fChecked/> No
      </span>
    </div>

    <div class="row">
      <span class="description">Disable Public Signup?</span><br />
      <span class="element">
        <input type='radio' name='ldapOptionsDisableSignup' value='1' $tDisableSignup/> Yes
        <input type='radio' name='ldapOptionsDisableSignup' value='0' $fDisableSignup/> No <br />
        <em>This would override all actions that take place with wp-signup.php.</em>
      </span>
    </div>

    <div class="row">
      <span class="description">Signup-Disabled Message:</span><br />
      <span class="element">
        <textarea name='ldapSignupMessage' id='ldapSignupMessage'>$ldapSignupMessage</textarea><br />
        <em>This is an alternate HTML message that would be displayed in place of any actions at wp-signup.php.</em>
      </span>
    </div>

    <div class="row">
      <span class="description">Lost-Password Message:</span><br />
      <span class="element">
        <textarea name='ldapGetPasswordMessage' id='ldapGetPasswordMessage'>$ldapGetPasswordMessage</textarea><br />
        <em>This is the error message that would be displayed when an LDAP-account user submits "Lost Password" requests.</em>
      </span>
    </div>

    
    <p class="submit"><input type="submit" name="ldapOptionsSave" value="Save" /></p>
    </fieldset>
  </form>
  </div>
LdapForm;
}

/**
 * Adds a sub menu to the Site Admin panel.  If the currently logged in user is
 * a site-admin, then this menu is created using the ldapOptionsPanel function.
 * Otherwise, nothing happens.
 * 
 * @return null - does not actively return a value
 */
function ldap_addmenu() {
  $objCurrUser = wp_get_current_user();
  $objUser = wp_cache_get($objCurrUser->id, 'users');
  if (function_exists('add_submenu_page') && is_site_admin($objUser->user_login)) {
    // does not use add_options_page, because it is site-wide configuration, 
    //  not blog-specific config, but side-wide 
    add_submenu_page('wpmu-admin.php', 'LDAP Authentication Options', 'LDAP Options', 9, basename(__FILE__), 'ldapOptionsPanel');
  }
}


// we have no desire to manage or manipulate the LDAP password, so the "lost 
//  password" action doesn't make sense for LDAP accounts; so work around it
add_action('retrieve_password', 'ldapRetrievePassword');
function ldapRetrievePassword($userName) {
  global $errors;
  
  $isLdapAuthOn = get_site_option('ldapAuth');
  
  // if not doing LDAP or it's a site admin, proceed as normal
  if (!$isLdapAuthOn || is_site_admin($userName) ) {
  	return true;
  }

  // otherwise, send an email with a configured message to the user
  // default is something like "Your information is stored in a central directory server.  
  //  You will need to contact the <site email>administrator to determine it."  
  else {
    // get the configurable error message:
    $getPwdMessage = get_site_option('ldapGetPasswordMessage');
        
    $errors['nonewpassword'] = __("<strong>ERROR</strong>: $getPwdMessage");
    login_header(__('Lost Password'));
    
    // HACK: Just shows a cut-n-paste of the login form as a workaround, since actions do not allow us to cease execution (e.g., by returning false), so need to display the form with the error message 
?>

<form name="lostpasswordform" id="lostpasswordform" action="wp-login.php?action=lostpassword" method="post">
  <p>
    <label><?php _e('Username:') ?><br />
    <input type="text" name="user_login" id="user_login" class="input" value="<?php echo attribute_escape(stripslashes($_POST['user_login'])); ?>" size="20" tabindex="10" /></label>
  </p>
  <p>
    <label><?php _e('E-mail:') ?><br />
    <input type="text" name="user_email" id="user_email" class="input" value="<?php echo attribute_escape(stripslashes($_POST['user_email'])); ?>" size="25" tabindex="20" /></label>
  </p>
<?php do_action('lostpassword_form'); ?>
  <p class="submit"><input type="submit" name="wp-submit" id="wp-submit" value="<?php _e('Get New Password &raquo;'); ?>" tabindex="100" /></p>
</form>
</div>

<ul>
<?php if (get_option('users_can_register')) : ?>
  <li><a href="<?php bloginfo('wpurl'); ?>/wp-login.php"><?php _e('Login') ?></a></li>
  <li><a href="<?php bloginfo('wpurl'); ?>/signup/"><?php _e('Register') ?></a></li>
  <li><a href="<?php bloginfo('url'); ?>/" title="<?php _e('Are you lost?') ?>"><?php printf(__('Back to %s'), get_bloginfo('title')); ?></a></li>
<?php else : ?>
  <li><a href="<?php bloginfo('url'); ?>/" title="<?php _e('Are you lost?') ?>"><?php printf(__('Back to %s'), get_bloginfo('title')); ?></a></li>
  <li><a href="<?php bloginfo('wpurl'); ?>/wp-login.php"><?php _e('Login') ?></a></li>
<?php endif; ?>
</ul>

</body>
</html>
<?php
    die();    
  }
}


/**
 * Returns false if LDAP authentication is turned on, the current user is a site
 * admin and the use being currently edited is a site admin.  Essentially is
 * indended to be used as a filter function on the WP filter
 * show_password_fields so as to hide the "new password" fields from appearing
 * for LDAP-managed accounts or for users that should not see the fields
 * regardless of access role.
 * 
 * Provides a means for accounts that are directly managed by WPMU (i.e., site
 * admins) to manage and update their passwords.
 * 
 * @param bool $isShowingPwdFields - the filtered value, expected from the
 * filter show_password_fields
 */
function wpmuLdapDisableLdapPassword($isShowingPwdFields) {
  global $userdata, $profileuser;
  get_currentuserinfo();

  $isLdapAuth = get_site_option('ldapAuth');
  
  // if LDAP is turned off, then just let it pass through untouched
  
  if ($isLdapAuth) {
    // even if LDAP auth is turned on, site admins still have their set-password 
    //  fields activated in their profiles
    
    // it's either an LDAP user being editied, or the logged in user shouldn't 
    //  have access anyway  
    if (!is_site_admin($userdata->user_login) || 
        !is_site_admin($profileuser->user_login))
      $isShowingPwdFields = false;
  } 
  
  return $isShowingPwdFields;
}

?>